<#import "utils.ftl" as utils>
<#assign rootEntity = entity.root>
<#-- Utility list to remember tarversed entities -->
<#assign traveresedEntityKeyList = [rootEntity.documentKey]>
<#-- Utility list to remember entities yet to be traversed -->
<#assign toTravereseEntityList = []>
<#-- Print CSV header row -->
<@printCsvHeadersRow />
<#-- Traverse the main entity -->
<@traverseEntity rootEntity />
<#-- Go throught the found referenced entities and traverse them. NOTE: the list can grow further as entities can reference further entities. Unclear whether this works well in FreeMarker -->
<#-- FreeMarker does not support do-while loops as such do this workaround with maxLoop -->
<#assign maxLoop=5>
<#list 1..maxLoop as x>
  <#-- Remove documents that have been already traversed -->
  <#assign toTravereseEntityList = utils.removeDocumentWitKey(toTravereseEntityList, traveresedEntityKeyList) >
  <#if !(toTravereseEntityList?has_content)>
    <#-- Stop looping if there is nothing to traverse -->
    <#break>
  </#if>
  <#-- Travers each entity from the list -->
  <#list toTravereseEntityList as entityItem>
    <#assign traveresedEntityKeyList= traveresedEntityKeyList + [entityItem.documentKey]>
    <@traverseEntity entityItem />
  </#list>
</#list>


<#----------------------------------------------->
<#-- Macro and function definitions start here -->
<#----------------------------------------------->

<#macro traverseEntity entityDoc>
  <#if utils.isCompositeEntityDocument(entityDoc)>
    <@traverseCompositeEntity entityDoc />
  <#else>
    <@traverseSimpleEntity entityDoc />
  </#if>
</#macro>

<#macro traverseSimpleEntity entityDoc>
  <@traverseDoc entityDoc entityDoc 0 />
</#macro>

<#macro traverseCompositeEntity entityDoc>
  <#local tableOfContents=iuclid.localizeTreeFor(entityDoc.documentType, entityDoc.submissionType, entityDoc.documentKey, utils.getInheritedTemplates(entityDoc), utils.getRelatedCategories(entityDoc), utils.getRelatedMixtures(entityDoc))>
  <@traverseToc tableOfContents entityDoc 0 />
</#macro>

<#macro traverseToc sectionNode entityDocument level=0>
  <#if sectionNode?has_content>
    <#-- iterate through the section documents under this section if there are any -->
    <#if sectionNode.content?has_content>
      <#list sectionNode.content as doc>
        <#-- traverse through the document's blocks and fields aka elements -->
        <@traverseDoc doc entityDocument level+1 sectionNode />
      </#list>
    </#if>
    <#-- iterate through the child sections of the node and do a recursive call -->
    <#if sectionNode?children?has_content>
      <#list sectionNode?children as child>
        <@traverseToc child entityDocument level+1 />
      </#list>
    </#if>
  </#if>
</#macro>

<#macro traverseDoc docElementNode entityDocument level=0 sectionNode="">
  <#if docElementNode?has_content>
    <@printIfAttachmentElementNode docElementNode entityDocument sectionNode />
    <#-- Check if the element node is a reference field (single or multiple) and remember referenced entities for later traversal in toTravereseEntityList -->
    <#assign toTravereseEntityList = utils.checkAndRememberReferencedEntity(docElementNode, toTravereseEntityList, traveresedEntityKeyList) />
    <#-- iterate through the child structure elements of this document element if there are any and do a recursive call -->
    <#if docElementNode?children?has_content>
      <#list docElementNode?children as child>
        <@traverseDoc child entityDocument level+1 sectionNode />
      </#list>
    </#if>
  </#if>
</#macro>

<#macro printIfAttachmentElementNode docElementNode entityDocument sectionNode>
  <#if docElementNode?node_type == "attachment">
    <#-- the string represenetation of the element node represents the attachment key -->
    <#local attachmentKey = docElementNode>
    <@printAttachmentField attachmentKey docElementNode entityDocument sectionNode />
  </#if>
  <#if docElementNode?node_type == "attachments">
     <#list docElementNode as item>
      <#-- the string represenetation of item represents the attachment key -->
      <#local attachmentKey = item>
      <@printAttachmentField attachmentKey docElementNode entityDocument sectionNode />
    </#list>
  </#if>
</#macro>

<#macro printAttachmentField attachmentKey fieldNode entityDocument sectionNode>
  <#local attachmentMetaData = iuclid.getMetadataForAttachment(attachmentKey)>
  <#local attachmentSize = "">
  <#if attachmentMetaData.size?has_content>
    <#local attachmentSize = attachmentMetaData.size?string["0.####"]>
  </#if>
  <#local sectionName = "">
  <#if sectionNode?has_content>
    <#if sectionNode.number?has_content>
      <#local sectionName = sectionNode.number + " ">
    </#if>
    <#local sectionName = sectionName + sectionNode.title>
  </#if>
  <#-- The root of the fieldNode is the node that represents the Document to which the field belongs to -->
  <#local sectionDoc = fieldNode?root>
  <#local sectionDocType = sectionDoc.documentType>
  <#if !utils.isEntityDocument(sectionDoc)>
    <#local sectionDocType = sectionDocType + "." + sectionDoc.documentSubType>
  </#if>
  <#local fieldPath = fieldNode._path>
  <#-- The field label -->
  <@iuclid.label for=fieldNode var="fieldLabel"/>
  <#-- The web URL pointing to the document -->
  <#assign docUrl=utils.getDocumentUrl(sectionDoc) />
  <#-- If the attachment belongs to a LITERATURE, print some extra information -->
  <#local literatureAuthor = "">
  <#local literatureTitle = "">
  <#local literatureYear = "">
  <#local literatureReportNo = "">
  <#local literatureReportDate = "">
  <#local literatureSource = "">
  <#if entityDocument.documentType == "LITERATURE">
    <#if entityDocument.GeneralInfo.Author?has_content>
      <#local literatureAuthor = entityDocument.GeneralInfo.Author>
    </#if>
    <#if entityDocument.GeneralInfo.Name?has_content>
      <#local literatureTitle = entityDocument.GeneralInfo.Name>
    </#if>
    <#if entityDocument.GeneralInfo.ReferenceYear?has_content>
      <#local literatureYear = entityDocument.GeneralInfo.ReferenceYear>
    </#if>
    <#if entityDocument.GeneralInfo.ReportNo?has_content>
      <#local literatureReportNo = entityDocument.GeneralInfo.ReportNo>
    </#if>
    <#if entityDocument.GeneralInfo.ReportDate?has_content>
      <#local literatureReportDate = entityDocument.GeneralInfo.ReportDate>
    </#if>
    <#if entityDocument.GeneralInfo.Source?has_content>
      <#local literatureSource = entityDocument.GeneralInfo.Source>
    </#if>
  </#if>
<@utils.csvValue attachmentMetaData.filename/>,<@utils.csvValue attachmentMetaData.remarks/>,${attachmentMetaData.mediaType},,${attachmentMetaData.md5},<@utils.csvValue fieldLabel/>,${entityDocument.documentKey.snapshotUuid},${entityDocument.documentType},<@utils.csvValue entityDocument.name/>,${entityDocument.documentKey},<@utils.csvValue sectionName/>,,<@utils.csvValue sectionDoc.name/>,${sectionDocType},<@utils.csvValue literatureAuthor/>,<@utils.csvValue literatureTitle/>,<@utils.csvValue literatureYear/>,<@utils.csvValue literatureReportNo/>,<@utils.csvValue literatureReportDate/>,<@utils.csvValue literatureSource/>,,,<@utils.csvValue docUrl/>,${attachmentSize},${fieldPath},${sectionDoc.documentKey},
</#macro>

<#macro printCsvHeadersRow>
"Attachment file name","Attachment remarks","Attachment media type","Upload date","MD5 checksum","Attachment field label","Dossier UUID","Linked to: Document Type","Linked to: Document name","Linked to: Document UUID","Parent: Section number and name","Parent: EU PP ToC","Parent document: Name",Parent document: Type,"Author","Title","Year","Report No.","Report date","Source","GLP","Data protection from parent","URL","Attachment file size","Attachment field path","Section document UUID"
</#macro>